--[[
Baggins_ClosetGnome

Author: Daphne Pfister <tiiroib02@sneakemail.com>

Thanks to Nargiddley for writing Baggins and Rabbit for writing ClosetGnome

License

Copyright (c) 2007 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


TODO:
Localize
Consider some sort of table compositing... should not be necassary though sets only time
tables are created/removed enmase is when a new set is added, or an old set is removed.

]]

Baggins_ClosetGnome = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0","AceEvent-2.0")

local Baggins_ClosetGnome = Baggins_ClosetGnome

local dewdrop = AceLibrary:HasInstance("Dewdrop-2.0") and AceLibrary("Dewdrop-2.0")
local items_table = {}
local sets_table = {}

local function linkToId(link)
	if link then
		return select(3, link:find("item:([%-%d]+:[%-%d:]+)"))
	end
	return nil
end

local function Matches(bag,slot,rule)
	local id = linkToId(GetContainerItemLink(bag, slot))
	if not id then return end

	local set = rule.set
	if not set then return end

	local invert = nil
	if set == "None" then
		set = "Any"
		invert = true
	end

	local res = items_table[id]
	if set ~= "Any" then res = res and res[set] end
	if invert then res = not res end
	return res
end

local function GetName(rule)
	return "ClosetGnome Set:"..(rule.set or "")
end

local function CleanRule(rule)
	rule.set = "Any"
end

local function DewDropOptions(rule, level, value)
	if level == 1 then
		dewdrop:AddLine("text", "ClosetGnome Sets", "isTitle", true)
		dewdrop:AddLine("text", "Any", "checked", rule.set == "Any","func",function(k) rule.set = "Any" Baggins:OnRuleChanged() end,"arg1","Any")
		dewdrop:AddLine("text", "None", "checked", rule.set == "None","func",function(k) rule.set = "None" Baggins:OnRuleChanged() end,"arg1","None")
		local setnames = {}
		for set in pairs(sets_table) do
			table.insert(setnames,set)
		end
		table.sort(setnames)
		for _,set in ipairs(setnames) do
			dewdrop:AddLine('text', set, "checked", rule.set == set,"func",function(k) rule.set = set Baggins:OnRuleChanged() end,"arg1",set)
		end
	end
end

function Baggins_ClosetGnome:OnEnable()
	self:CacheSets()

	self:RegisterEvent("ClosetGnome_AddSet", "CacheSets")
	self:RegisterEvent("ClosetGnome_UpdateSet", "CacheSets")
	self:RegisterEvent("ClosetGnome_DeleteSet", "CacheSets")

	Baggins:AddCustomRule("ClosetGnome", {
		DisplayName = "ClosetGnome",
		Description = "Filter by ClosetGnome set",
		Matches = Matches,
		GetName = GetName,
		CleanRule = CleanRule,
		DewDropOptions = DewDropOptions
	})
end

function Baggins_ClosetGnome:CacheSets()
	local i,set

	local sets = ClosetGnome.db.char.set

	items_table = {}
	sets_table = {}
	for setname, set in pairs(sets) do
		sets_table[setname] = true
		for i=0,19 do
			local setitem = linkToId(set[i])
			if setitem then
				if not items_table[setitem] then items_table[setitem] = {} end
				items_table[setitem][setname] = true
			end
		end
	end
	if Baggins then Baggins:ForceFullUpdate() end
end

